import * as grpc from 'grpc';
import { ClientGRPC } from '../grpc/grpcClient';
import { getKey, logger, UserInfo, commonAppConst, generateJWT } from 'skyeye-svc-common-utils';
import { GrpcClientInstance } from '../interfaces/grpcClientInstance';
import { GrpcServiceResponse } from '../interfaces/grpcServiceResponse';
import { grpcConst } from '../utils/grpcConst';
import { GrpcListRequest } from '../interfaces/grpcListRequest';
const ProtoPath = __dirname + grpcConst.protoPath + grpcConst.proto.campaign;

export class CampaignClient {
    private static instance: CampaignClient;
    private static campaignServer: any;
    private static metaData: grpc.Metadata;
    private static options: grpc.CallOptions;
    private constructor() { }
    static async getInstance(userInfo: UserInfo): Promise<CampaignClient> {
        if (!CampaignClient.instance) {
            CampaignClient.instance = new CampaignClient();
            let clientGrpc: ClientGRPC = new ClientGRPC();
            let clientInit: GrpcClientInstance = await clientGrpc.initClient(ProtoPath);

            let CampaignService: any = grpc.loadPackageDefinition(clientInit.loadPackageDefinition);
            CampaignService = CampaignService.hs[grpcConst.service.campaign].Service;

            let clientConnection = await getKey(grpcConst.hostName.campaign) || grpcConst.defaultGRPCAddress;
            this.campaignServer = new CampaignService(`${clientConnection}`, grpc.credentials.createInsecure());
            logger.info(`started ${grpcConst.hostName.campaign} connection : ${clientConnection}`);
        }
        CampaignClient.metaData = new grpc.Metadata();
        if (userInfo) {
            let token: string = `${commonAppConst.httpHeader.BEARER} ${await generateJWT(userInfo)}`;
            CampaignClient.metaData.add(commonAppConst.httpHeader.AUTHORIZATION, token);
        }
        let deadline = Number(await getKey(grpcConst.deadline.short));
        deadline = new Date().setSeconds(new Date().getSeconds() + deadline);
        CampaignClient.options = { deadline: deadline };
        return CampaignClient.instance;
    }

    async root(): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.root`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.root(
                {},
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.root`)
                })
        });
    }

    async campaignEventCreate(name: string, description: string, region: string, bannerUrl: string,
        period: number, publishStartDate: string, publishEndDate: string, isPublic: boolean,
        isActive: boolean, campaignEventScoutAIs: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventCreate`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventCreate(
                {
                    name: name,
                    description: description,
                    region: region,
                    bannerUrl: bannerUrl,
                    period: period,
                    publishStartDate: publishStartDate,
                    publishEndDate: publishEndDate,
                    isPublic: isPublic,
                    isActive: isActive,
                    campaignEventScoutAIs: campaignEventScoutAIs
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventCreate`)
                })
        });
    }

    async campaignEventDelete(guid: string, deletedStatus: number): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventDelete`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventDelete(
                { guid: guid, deletedStatus: deletedStatus },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventDelete`)
                })
        });
    }

    async campaignEventList(name: string, region: string, publishStartDate: string, publishEndDate: string, isPublic: boolean,
        isActive: boolean, includeAIs: boolean, status: number[], emailTemplates: string[], listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventList`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventList(
                {
                    name: name,
                    region: region,
                    publishStartDate: publishStartDate,
                    publishEndDate: publishEndDate,
                    isPublic: isPublic,
                    isActive: isActive,
                    status: status,
                    includeAIs: includeAIs,
                    emailTemplates: emailTemplates,
                    ...listProps
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventList`)
                })
        });
    }


    async campaignEventRetrieve(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventRetrieve`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventRetrieve(
                { guid: guid },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventRetrieve`)
                })
        });
    }

    async campaignEventRetrieveByIds(guids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventRetrieveByIds`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventRetrieveByIds(
                { guids: guids },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventRetrieveByIds`)
                })
        });
    }

    async campaignEventUpdate(guid: string, name: string, description: string, region: string, bannerUrl: string,
        period: number, publishStartDate: string, publishEndDate: string, isPublic: boolean,
        isActive: boolean, campaignEventScoutAIs: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventUpdate`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventUpdate(
                {
                    guid: guid,
                    name: name,
                    description: description,
                    region: region,
                    bannerUrl: bannerUrl,
                    period: period,
                    publishStartDate: publishStartDate,
                    publishEndDate: publishEndDate,
                    isPublic: isPublic,
                    isActive: isActive,
                    campaignEventScoutAIs: campaignEventScoutAIs
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventUpdate`)
                })
        });
    }

    async campaignEventUserUpdate(campaignEventGuid: string, campaignUserGuid: string, status: number, digestStartDate: string, digestEndDate: string): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventUserUpdate`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventUserUpdate(
                {
                    campaignEventGuid: campaignEventGuid,
                    campaignUserGuid: campaignUserGuid,
                    status: status,
                    digestStartDate: digestStartDate,
                    digestEndDate: digestEndDate
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventUserUpdate`)
                })
        });
    }

    async campaignEventUserUnsubscribe(campaignEventGuid: string, campaignUserGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventUserUnsubscribe`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventUserUnsubscribe(
                {
                    campaignEventGuid: campaignEventGuid,
                    campaignUserGuid: campaignUserGuid
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventUserUnsubscribe`)
                })
        });
    }

    async campaignEventUserList(campaignEventGuid: string, campaignUserGuid: string, status: number[], deletedStatus: number[], listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventUserList`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventUserList(
                {
                    campaignEventGuid: campaignEventGuid,
                    campaignUserGuid: campaignUserGuid,
                    status: status,
                    deletedStatus: deletedStatus,
                    ...listProps
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventUserList`)
                })
        });
    }

    async campaignEventUserRetrieveByCampaignGuids(campaignEventGuids: string[], status: number[], deletedStatus: number[], digestDate: string, listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignEventUserRetrieveByCampaignGuids`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignEventUserRetrieveByCampaignGuids(
                {
                    campaignEventGuids: campaignEventGuids,
                    status: status,
                    deletedStatus: deletedStatus,
                    digestDate: digestDate,
                    ...listProps
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignEventUserRetrieveByCampaignGuids`)
                })
        });
    }

    async campaignUserCreate(email: string, firstName: string, lastName: string, contactNumber: string, company: string, campaignEventGuids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignUserCreate`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignUserCreate(
                {
                    email: email,
                    firstName: firstName,
                    lastName: lastName,
                    contactNumber: contactNumber,
                    company: company,
                    campaignEventGuids: campaignEventGuids
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignUserCreate`)
                })
        });
    }

    async campaignUserDelete(guid: string, deletedStatus: number): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignUserDelete`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignUserDelete(
                { guid: guid, deletedStatus: deletedStatus },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignUserDelete`)
                })
        });
    }

    async campaignUserInitAccount(guid: string, status: number, subscriptionGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignUserInitAccount`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignUserInitAccount(
                { guid: guid, status: status, subscriptionGuid: subscriptionGuid },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignUserInitAccount`)
                })
        });
    }

    async campaignUserList(email: string, eventGuid: string, userStatus: number, status: number[], listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignUserList`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignUserList(
                { email: email, eventGuid: eventGuid, userStatus: userStatus, status: status, ...listProps },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignUserList`)
                })
        });
    }

    async campaignUserRetrieveByIdsReq(guids: string[], userStatus: number[], status: number[], digestStartDate: string, digestEndDate: string, listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignUserRetrieveByIdsReq`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignUserList(
                { guids: guids, userStatus: userStatus, status: status, digestStartDate: digestStartDate, digestEndDate: digestEndDate, ...listProps },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignUserRetrieveByIdsReq`)
                })
        });
    }


    async compaignUserRequestAccount(requestType: string, requestKey: string): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.compaignUserRequestAccount`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.compaignUserRequestAccount(
                { requestType: requestType, requestKey: requestKey },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.compaignUserRequestAccount`)
                })
        });
    }

    async campaignUserRetrieve(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignUserRetrieve`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignUserRetrieve(
                { guid: guid },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignUserRetrieve`)
                })
        });
    }

    async campaignUserUpdate(guid: string, email: string, firstName: string, lastName: string, contactNumber: string, company: string, campaignEventGuids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignUserUpdate`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignUserUpdate(
                {
                    guid: guid,
                    email: email,
                    firstName: firstName,
                    lastName: lastName,
                    contactNumber: contactNumber,
                    company: company,
                    campaignEventGuids: campaignEventGuids
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignUserUpdate`)
                })
        });
    }

    async campaignUserNotifyActivate(email:string): Promise<GrpcServiceResponse> {
        logger.info(`start CampaignClient.campaignUserNotifyActivate`)
        return new Promise((resolve) => {
            CampaignClient.campaignServer.campaignUserNotifyActivate(
                {
                    email: email
                },
                CampaignClient.metaData,
                CampaignClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end CampaignClient.campaignUserNotifyActivate`)
                })
        });
    }
}


