import * as grpc from 'grpc';
import { ClientGRPC } from '../grpc/grpcClient';
import { getKey, logger, UserInfo, generateJWT, commonAppConst } from 'skyeye-svc-common-utils';
import { GrpcClientInstance } from '../interfaces/grpcClientInstance';
import { GrpcServiceResponse } from '../interfaces/grpcServiceResponse';
import { grpcConst } from '../utils/grpcConst';
import { GrpcListRequest } from '../interfaces/grpcListRequest';
const ProtoPath = __dirname + grpcConst.protoPath + grpcConst.proto.organisation;

export class OrganisationClient {
    private static instance: OrganisationClient;
    private static organisationServer: any;
    private static metaData: grpc.Metadata;
    private static options: grpc.CallOptions;
    private constructor() { }
    public static async getInstance(userInfo: UserInfo): Promise<OrganisationClient> {
        if (!OrganisationClient.instance) {
            OrganisationClient.instance = new OrganisationClient();
            let clientGrpc: ClientGRPC = new ClientGRPC();
            let clientInit: GrpcClientInstance = await clientGrpc.initClient(ProtoPath);

            let OrganisationService: any = grpc.loadPackageDefinition(clientInit.loadPackageDefinition);
            OrganisationService = OrganisationService.hs[grpcConst.service.organisation].Service;

            let clientConnection = await getKey(grpcConst.hostName.organisation) || grpcConst.defaultGRPCAddress;
            this.organisationServer = new OrganisationService(`${clientConnection}`, grpc.credentials.createInsecure());
            logger.info(`started ${grpcConst.hostName.organisation} connection : ${clientConnection}`);
        }

        OrganisationClient.metaData = new grpc.Metadata();
        if (userInfo) {
            let token: string = `${commonAppConst.httpHeader.BEARER} ${await generateJWT(userInfo)}`;
            OrganisationClient.metaData.add(commonAppConst.httpHeader.AUTHORIZATION, token);
        }
        let deadline = Number(await getKey(grpcConst.deadline.short));
        deadline = new Date().setSeconds(new Date().getSeconds() + deadline);
        OrganisationClient.options = { deadline: deadline };
        return OrganisationClient.instance;
    }

    async root(): Promise<GrpcServiceResponse> {
        logger.info(`start OrganisationClient.root`)
        return new Promise((resolve) => {
            OrganisationClient.organisationServer.root(
                {},
                OrganisationClient.metaData,
                OrganisationClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end OrganisationClient.root`)
                })
        });
    }

    async organisationCreate(name: string, description: string, phone: string, email: string, location: string): Promise<GrpcServiceResponse> {
        logger.info(`start OrganisationClient.organisationCreate`)
        return new Promise((resolve) => {
            OrganisationClient.organisationServer.organisationCreate(
                { name: name, description: description, phone: phone, email: email, location: location }, OrganisationClient.metaData,
                OrganisationClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end OrganisationClient.organisationCreate`)
                })
        })
    }

    async organisationDelete(guid: string, deletedStatus?: number): Promise<GrpcServiceResponse> {
        logger.info(`start OrganisationClient.organisationDelete`)
        return new Promise((resolve) => {
            OrganisationClient.organisationServer.organisationDelete(
                { guid: guid, deletedStatus: deletedStatus }, OrganisationClient.metaData,
                OrganisationClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end OrganisationClient.organisationDelete`)
                })
        })
    }

    async organisationRetrieve(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start OrganisationClient.organisationRetrieve`)
        return new Promise((resolve) => {
            OrganisationClient.organisationServer.organisationDelete(
                { guid: guid }, OrganisationClient.metaData,
                OrganisationClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end OrganisationClient.organisationRetrieve`)
                })
        })
    }

    async organisationList(guid: string = null, name: string = null, phone: string = null, email: string = null, deletedStatus: number = null, listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start OrganisationClient.organisationList`)
        return new Promise((resolve) => {
            OrganisationClient.organisationServer.organisationList(
                { guid: guid, name: name, phone: phone, email: email, deletedStatus: deletedStatus, ...listProps }, OrganisationClient.metaData,
                OrganisationClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end OrganisationClient.organisationList`)
                })
        })
    }

    async organisationRetrieveByIds(guids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start OrganisationClient.organisationRetrieveByIds`)
        return new Promise((resolve) => {
            OrganisationClient.organisationServer.organisationRetrieveByIds(
                { guids: guids }, OrganisationClient.metaData,
                OrganisationClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end OrganisationClient.organisationRetrieveByIds`)
                })
        })
    }

    async organisationUpdate(guid: string, name: string, description: string, phone: string, email: string, location: string): Promise<GrpcServiceResponse> {
        logger.info(`start OrganisationClient.organisationUpdate`)
        return new Promise((resolve) => {
            OrganisationClient.organisationServer.organisationUpdate(
                { guid: guid, name: name, description: description, phone: phone, email: email, location: location }, OrganisationClient.metaData,
                OrganisationClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end OrganisationClient.organisationUpdate`)
                })
        })
    }
}


