import * as grpc from 'grpc';
import { ClientGRPC } from '../grpc/grpcClient';
import { getKey, logger, UserInfo, commonAppConst, generateJWT } from 'skyeye-svc-common-utils';
import { GrpcClientInstance } from '../interfaces/grpcClientInstance';
import { GrpcServiceResponse } from '../interfaces/grpcServiceResponse';
import { grpcConst } from '../utils/grpcConst';
import { GrpcListRequest } from '../interfaces/grpcListRequest';
import { constructClientHandler } from '../test/apiTest';
const ProtoPath = __dirname + grpcConst.protoPath + grpcConst.proto.rbac;

export class RbacClient {
    private static instance: RbacClient;
    private static rbacServer: any;
    private static metaData: grpc.Metadata;
    private static options: grpc.CallOptions;
    private constructor() { }
    static async getInstance(userInfo: UserInfo): Promise<RbacClient> {

        // this is a proxy for api testing, for more details, see SAENG-295
        if (process.env.NODE_ENV === 'test') {

            if (!RbacClient.instance) {
                // proxy functions with handler
                const handler = constructClientHandler(RbacClient.name);
                RbacClient.instance = new Proxy(new RbacClient(), handler);
            }

            return RbacClient.instance;
        }

        if (!RbacClient.instance) {
            RbacClient.instance = new RbacClient();
            let clientGrpc: ClientGRPC = new ClientGRPC();
            let clientInit: GrpcClientInstance = await clientGrpc.initClient(ProtoPath);

            let RbacService: any = grpc.loadPackageDefinition(clientInit.loadPackageDefinition);
            RbacService = RbacService.hs[grpcConst.service.rbac].Service;

            let clientConnection = await getKey(grpcConst.hostName.rbac) || grpcConst.defaultGRPCAddress;
            this.rbacServer = new RbacService(`${clientConnection}`, grpc.credentials.createInsecure());
            logger.info(`started ${grpcConst.hostName.rbac} connection : ${clientConnection}`);
        }
        RbacClient.metaData = new grpc.Metadata();
        if (userInfo){
            let token: string = `${commonAppConst.httpHeader.BEARER} ${await generateJWT(userInfo)}`;
            RbacClient.metaData.add(commonAppConst.httpHeader.AUTHORIZATION, token);
        }
        let deadline = Number(await getKey(grpcConst.deadline.short));
        deadline = new Date().setSeconds(new Date().getSeconds() + deadline);
        RbacClient.options = { deadline: deadline };
        return RbacClient.instance;
    }

    async root(): Promise<GrpcServiceResponse> {
        logger.info(`start RbacClient.root`)
        return new Promise((resolve) => {
            RbacClient.rbacServer.root(
                {},
                RbacClient.metaData,
                RbacClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end RbacClient.root`)
                })
        });
    }

    async roleCreate(name: string, description: string, rolePermissions: string[], organisationGuid?: string): Promise<GrpcServiceResponse> {
        logger.info(`start RbacClient.roleCreate`)
        return new Promise((resolve) => {
            RbacClient.rbacServer.roleCreate(
                {
                    name: name,
                    description: description,
                    organisationGuid: organisationGuid,
                    rolePermissions: rolePermissions
                },
                RbacClient.metaData,
                RbacClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end RbacClient.roleCreate`)
                })
        });
    }

    async roleRetrieve(guid: string, organisationGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start RbacClient.roleRetrieve`)
        return new Promise((resolve) => {
            RbacClient.rbacServer.roleRetrieve(
                {
                    guid: guid,
                    organisationGuid: organisationGuid
                },
                RbacClient.metaData,
                RbacClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end RbacClient.roleRetrieve`)
                })
        });
    }

    async roleList(roleName: string, organisationGuid: string, listProps: GrpcListRequest, status?: number[]): Promise<GrpcServiceResponse> {
        logger.info(`start RbacClient.roleList`)
        return new Promise((resolve) => {
            RbacClient.rbacServer.roleList(
                {
                    roleName: roleName,
                    status: status,
                    organisationGuid: organisationGuid,
                    ...listProps
                },
                RbacClient.metaData,
                RbacClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end RbacClient.roleList`)
                })
        });
    }

    async roleRetrieveByIds(roleGuids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start RbacClient.roleRetrieveByIds`)
        return new Promise((resolve) => {
            RbacClient.rbacServer.roleRetrieveByIds(
                { roleGuids: roleGuids },
                RbacClient.metaData,
                RbacClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end RbacClient.roleRetrieveByIds`)
                })
        });
    }

    async roleUpdate(guid: string, name: string, description: string, organisationGuid?: string, rolePermissions?: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start RbacClient.roleUpdate`)
        return new Promise((resolve) => {
            RbacClient.rbacServer.roleUpdate(
                {
                    guid: guid,
                    name: name,
                    description: description,
                    organisationGuid: organisationGuid,
                    rolePermissions: rolePermissions
                },
                RbacClient.metaData,
                RbacClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end RbacClient.roleUpdate`)
                })
        });
    }

    async roleDelete(guid: string, organisationGuid?: string, deletedStatus?: number): Promise<GrpcServiceResponse> {
        logger.info(`start RbacClient.roleDelete`)
        return new Promise((resolve) => {
            RbacClient.rbacServer.roleDelete(
                {
                    guid: guid,
                    organisationGuid: organisationGuid,
                    deletedStatus: deletedStatus
                },
                RbacClient.metaData,
                RbacClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end RbacClient.roleDelete`)
                })
        });
    }
}


