import * as grpc from 'grpc';
import { ClientGRPC } from '../grpc/grpcClient';
import { getKey, logger, UserInfo, generateJWT, commonAppConst } from 'skyeye-svc-common-utils';
import { GrpcClientInstance } from '../interfaces/grpcClientInstance';
import { GrpcServiceResponse } from '../interfaces/grpcServiceResponse';
import { grpcConst } from '../utils/grpcConst';
import { GrpcListRequest } from '../interfaces/grpcListRequest';
const ProtoPath = __dirname + grpcConst.protoPath + grpcConst.proto.scoutai;

export class ScoutAIClient {
    private static instance: ScoutAIClient;
    private static scoutAIServer: any;
    private static metaData: grpc.Metadata;
    private static options: grpc.CallOptions;
    private constructor() { }
    public static async getInstance(userInfo: UserInfo): Promise<ScoutAIClient> {
        if (!ScoutAIClient.instance) {
            ScoutAIClient.instance = new ScoutAIClient();
            let clientGrpc: ClientGRPC = new ClientGRPC();
            let clientInit: GrpcClientInstance = await clientGrpc.initClient(ProtoPath);

            let ScoutAIService: any = grpc.loadPackageDefinition(clientInit.loadPackageDefinition);
            ScoutAIService = ScoutAIService.hs[grpcConst.service.scoutai].Service;

            let clientConnection = await getKey(grpcConst.hostName.scoutai) || grpcConst.defaultGRPCAddress;
            this.scoutAIServer = new ScoutAIService(`${clientConnection}`, grpc.credentials.createInsecure());
            logger.info(`started ${grpcConst.hostName.scoutai} connection : ${clientConnection}`);
        }

        ScoutAIClient.metaData = new grpc.Metadata();
        if (userInfo) {
            let token: string = `${commonAppConst.httpHeader.BEARER} ${await generateJWT(userInfo)}`;
            ScoutAIClient.metaData.add(commonAppConst.httpHeader.AUTHORIZATION, token);
        }
        let deadline = Number(await getKey(grpcConst.deadline.mid));
        deadline = new Date().setSeconds(new Date().getSeconds() + deadline);
        ScoutAIClient.options = { deadline: deadline };
        return ScoutAIClient.instance;
    }

    async root(): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.root`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.root(
                {},
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.root`)
                })
        });
    }

    async createScoutAI(name: string, description: string = null, userGuid: string = null): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.createScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.createScoutAI(
                { name: name, description: description, userGuid: userGuid },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.createScoutAI`)
                })
        });
    }

    async deleteScoutAI(guid: string, deletedStatus: number): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.deleteScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.deleteScoutAI(
                { guid: guid, deletedStatus: deletedStatus },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.deleteScoutAI`)
                })
        });
    }

    async listScoutAI(userGuid: string, name: string, scoutType: string[], deletedStatus: number[], isPublic: boolean, isPagination: boolean, listProps: GrpcListRequest, skipStats?: Boolean): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.listScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.listScoutAI(
                {userGuid: userGuid, name: name, scoutType: scoutType, deletedStatus: deletedStatus, isPublic:isPublic, isPagination:isPagination, ...listProps, skipStats:skipStats },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.listScoutAI`)
                })
        });
    }

    async retrieveScoutAI(guid: string, deletedStatus: number): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.retrieveScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.retrieveScoutAI(
                { guid: guid, deletedStatus: deletedStatus },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.retrieveScoutAI`)
                })
        });
    }

    async retrieveByIdsScoutAI(guids: string[], deletedStatus: number[], scoutAIType: string, isPagination: boolean, listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.retrieveByIdsScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.retrieveByIdsScoutAI(
                { guids: guids, deletedStatus: deletedStatus, scoutAIType: scoutAIType, isPagination: isPagination, ...listProps },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.retrieveByIdsScoutAI`)
                })
        });
    }

    async updateScoutAI(guid: string, name: string, description: string, isPublic: boolean): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.updateScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.updateScoutAI(
                { guid: guid, name: name, description: description, isPublic: isPublic },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.updateScoutAI`)
                })
        });
    }

    async graduateScoutAI(guids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.graduateScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.graduateScoutAI(
                { guids: guids },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.graduateScoutAI`)
                })
        });
    }

    async shareScoutAI(scoutAIGuids: string[], emails: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.shareScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.shareScoutAI(
                { scoutAIGuids: scoutAIGuids, emails: emails },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.shareScoutAI`)
                })
        });
    }

    async continuousTrainResult(scoutAIGuid: string, articlels: []): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.continuousTrainResult`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.continuousTrainResult(
                { scoutAIGuid: scoutAIGuid, articlels: articlels },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.continuousTrainResult`)
                })
        });
    }

    async initialTrainResult(scoutAIGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.initialTrainResult`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.initialTrainResult(
                { scoutAIGuid: scoutAIGuid },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.initialTrainResult`)
                })
        });
    }

    async training(scoutAIGuid: string, articlels: []): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.training`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.training(
                { scoutAIGuid: scoutAIGuid, articlels: articlels },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.training`)
                })
        });
    }

    async retrieveScoutAIResult(scoutAIGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.retrieveScoutAIResult`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.retrieveScoutAIResult(
                { scoutAIGuid: scoutAIGuid },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.retrieveScoutAIResult`)
                })
        });
    }

    async resultContentRetrieve(scoutAIGuid: string, searchStr: string, isPagination: boolean, skip: number, select: number): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.resultContentRetrieve`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.resultContentRetrieve(
                { scoutAIGuid: scoutAIGuid, searchStr: searchStr, isPagination: isPagination, skip: skip, select: select },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.resultContentRetrieve`)
                })
        });
    }

    async articleResult(entityGuid: string, scoutAIGuids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.articleResult`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.articleResult(
                { entityGuid: entityGuid, scoutAIGuids: scoutAIGuids },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.articleResult`)
                })
        });
    }

    async articleTrainingResult(entityGuid: string, entityName: string, entityType: string, scoutAIs: any[]): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.articleTrainingResult`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.articleResult(
                { entityGuid: entityGuid, entityName: entityName, entityType: entityType, scoutAIs: scoutAIs },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.articleTrainingResult`)
                })
        });
    }

    async acceptShareScoutAI(resetKey: string, resetType: string): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.acceptShareScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.acceptShareScoutAI(
                { resetKey: resetKey, resetType: resetType },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.acceptShareScoutAI`)
                })
        });
    }

    async untrainedScoutAIRetrieve(entityGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.untrainedScoutAIRetrieve`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.untrainedScoutAIRetrieve(
                { entityGuid: entityGuid },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.untrainedScoutAIRetrieve`)
                })
        });
    }

    async searchUntrainContentScoutAI(scoutAIGuid: string, searchStr: string, aggregation: boolean): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.searchUntrainContentScoutAI`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.searchUntrainContentScoutAI(
                { scoutAIGuid: scoutAIGuid, searchStr: searchStr, aggregation: aggregation },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.searchUntrainContentScoutAI`)
                })
        });
    }

    async scoutAIVisualisation(guid: string, dim: number): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.scoutAIVisualisation`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.scoutAIVisualisation(
                { guid: guid, dim: dim },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.scoutAIVisualisation`)
                })
        });
    }

    async createTaasConfig(name: string, description: string, userGuid: string, scoutAIGuids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.createTaasConfig`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.createTaasConfig(
                { name: name, description: description, userGuid: userGuid, scoutAIGuids: scoutAIGuids },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.createTaasConfig`)
                })
        });
    }

    async deleteTaasConfig(guid: string, deletedStatus: number): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.deleteTaasConfig`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.deleteTaasConfig(
                { guid: guid, deletedStatus: deletedStatus },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.deleteTaasConfig`)
                })
        });
    }

    async retrieveTaasConfig(guid: string, deletedStatus: number): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.retrieveTaasConfig`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.retrieveTaasConfig(
                { guid: guid, deletedStatus: deletedStatus },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.retrieveTaasConfig`)
                })
        });
    }

    async retrieveByIdsTaasConfig(guids: string[], deletedStatus: number[]): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.retrieveByIdsTaasConfig`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.retrieveByIdsTaasConfig(
                { guids: guids, deletedStatus: deletedStatus },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.retrieveByIdsTaasConfig`)
                })
        });
    }

    async listTaasConfig(name: string, deletedStatus: number[], listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.listTaasConfig`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.listTaasConfig(
                { name: name, deletedStatus: deletedStatus, ...listProps },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.listTaasConfig`)
                })
        });
    }

    async updateTaasConfig(guid: string, name: string, description: string, scoutAIGuids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.updateTaasConfig`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.updateTaasConfig(
                { guid: guid, name: name, description: description, scoutAIGuids: scoutAIGuids },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.updateTaasConfig`)
                })
        });
    }

    async retrieveScoutAiByUserGuids(userGuids: string[], scoutType: string[], deletedStatus: number[], isPagination: boolean, listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.retrieveScoutAiByUserGuids`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.retrieveScoutAiByUserGuids(
                { userGuids: userGuids, scoutType: scoutType, deletedStatus: deletedStatus, isPagination: isPagination, ...listProps },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.retrieveScoutAiByUserGuids`)
                })
        });
    }

    async chooseActiveTaasConfig(guid: string, userGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.chooseActiveTaasConfig`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.chooseActiveTaasConfig(
                { guid: guid, userGuid: userGuid },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.chooseActiveTaasConfig`)
                })
        });
    }
    async nerlCorrectionSubmit(entityGuid: string, title: string, body:string, annotationJson: any[]): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.nerlCorrectionSubmit`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.nerlCorrectionSubmit(
                { entityGuid: entityGuid, title: title, body: body, annotationJson: annotationJson },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.nerlCorrectionSubmit`)
                })
        });
    }
    async retrieveNerlCorrectionByEntityGuid(entityGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.retrieveNerlCorrectionByEntityGuid`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.retrieveNerlCorrectionByEntityGuid(
                { entityGuid: entityGuid },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.retrieveNerlCorrectionByEntityGuid`)
                })
        });
    }
    async nerlCorrectionDelete(entityGuid: string, deletedStatus: number): Promise<GrpcServiceResponse> {
        logger.info(`start ScoutAIClient.nerlCorrectionDelete`)
        return new Promise((resolve) => {
            ScoutAIClient.scoutAIServer.nerlCorrectionDelete(
                { entityGuid: entityGuid, deletedStatus: deletedStatus },
                ScoutAIClient.metaData,
                ScoutAIClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end ScoutAIClient.nerlCorrectionDelete`)
                })
        });
    }

}


