import * as grpc from 'grpc';
import { ClientGRPC } from '../grpc/grpcClient';
import { getKey, logger, UserInfo, commonAppConst, generateJWT } from 'skyeye-svc-common-utils';
import { GrpcClientInstance } from '../interfaces/grpcClientInstance';
import { GrpcServiceResponse } from '../interfaces/grpcServiceResponse';
import { grpcConst } from '../utils/grpcConst';
import { GrpcContentListReq, GrpcCompanyListReq } from '../models/search';
const ProtoPath = __dirname + grpcConst.protoPath + grpcConst.proto.search;

export class SearchClient {
    private static instance: SearchClient;
    private static searchServer: any;
    private static metaData: grpc.Metadata;
    private static options: grpc.CallOptions;
    private constructor() { }
    static async getInstance(userInfo: UserInfo): Promise<SearchClient> {
        if (!SearchClient.instance) {
            SearchClient.instance = new SearchClient();
            let clientGrpc: ClientGRPC = new ClientGRPC();
            let clientInit: GrpcClientInstance = await clientGrpc.initClient(ProtoPath);

            let SearchService: any = grpc.loadPackageDefinition(clientInit.loadPackageDefinition);
            SearchService = SearchService.hs[grpcConst.service.search].Service;

            let clientConnection = await getKey(grpcConst.hostName.search) || grpcConst.defaultGRPCAddress;
            this.searchServer = new SearchService(`${clientConnection}`, grpc.credentials.createInsecure());
            logger.info(`started ${grpcConst.hostName.search} connection : ${clientConnection}`);
        }
        SearchClient.metaData = new grpc.Metadata();
        if (userInfo) {
            let token: string = `${commonAppConst.httpHeader.BEARER} ${await generateJWT(userInfo)}`;
            SearchClient.metaData.add(commonAppConst.httpHeader.AUTHORIZATION, token);
        }
        let deadline = Number(await getKey(grpcConst.deadline.long));
        deadline = new Date().setSeconds(new Date().getSeconds() + deadline);
        SearchClient.options = { deadline: deadline };
        return SearchClient.instance;
    }

    async root(): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.root`)
        return new Promise((resolve) => {
            SearchClient.searchServer.root(
                {},
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.root`)
                })
        });
    }

    async contentList(grpcContentListReq: GrpcContentListReq, aggregation: boolean = false, ignoreHighlight: boolean = false, allAiTagging: boolean = false, fields?: string[], order?: {attr: string, orderBy: string}, scoutAiFiltering?: boolean): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.contentList`)
        return new Promise((resolve) => {
            SearchClient.searchServer.contentList(
                {
                    ...grpcContentListReq,
                    aggregation,
                    ignoreHighlight,
                    allAiTagging,
                    fields,
                    order,
                    scoutAiFiltering
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.contentList`)
                })
        });
    }

    async mentionContentCount(guid: string, start: string, end: string): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.mentionContentCount`)
        return new Promise((resolve) => {
            SearchClient.searchServer.mentionContentCount(
                {
                    guid: guid,
                    start: start,
                    end: end
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.mentionContentCount`)
                })
        });
    }

    async contentTabs(companyGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.contentTabs`)
        return new Promise((resolve) => {
            SearchClient.searchServer.contentTabs(
                {
                    companyGuid: companyGuid
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.contentTabs`)
                })
        });
    }

    async contentRetrieveByIds(guids: string[], searchStr: string = '*', skip: number = null, select: number = null, includeIsPrimaryFalse = false): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.contentRetrieveByIds`)
        return new Promise((resolve) => {
            SearchClient.searchServer.contentRetrieveByIds(
                {
                    guids: guids,
                    searchStr: searchStr,
                    skip: skip,
                    select: select,
                    includeIsPrimaryFalse: includeIsPrimaryFalse
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.contentRetrieveByIds`)
                })
        });
    }

    async contentAggregateByMentionedCompany(mentionedCompanies: string[], start: string, end: string, coMentionRequired: number, coMentionFacetCount: number): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.contentAggregateByMentionedCompany`)
        return new Promise((resolve) => {
            SearchClient.searchServer.contentAggregateByMentionedCompany(
                {
                    mentionedCompanies,
                    start,
                    end,
                    coMentionRequired,
                    coMentionFacetCount
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.contentAggregateByMentionedCompany`)
                })
        });
    }

    async companyList(grpcCompanyListReq: GrpcCompanyListReq, aggregation: boolean = false, fields?:string[]): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.companyList`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyList(
                {
                    /* 
                        Adding clause to check if nullable param has value,
                        if value is displayed need to turn in into obj.
                        refer to : https://googleapis.dev/nodejs/analytics-admin/latest/google.protobuf.DoubleValue.html
                    */
                    skip: grpcCompanyListReq.skip,
                    select: grpcCompanyListReq.select,
                    order: grpcCompanyListReq.order,
                    orderBy: grpcCompanyListReq.orderBy,
                    searchStr: grpcCompanyListReq.searchStr,
                    sectors: grpcCompanyListReq.sectors,
                    countries: grpcCompanyListReq.countries,
                    naics: grpcCompanyListReq.naics,
                    listingStatus: grpcCompanyListReq.listingStatus,
                    numOfEmployeesFrom: grpcCompanyListReq.numOfEmployeesFrom ? { value: grpcCompanyListReq.numOfEmployeesFrom } : null,
                    numOfEmployeesTo: grpcCompanyListReq.numOfEmployeesTo ? { value: grpcCompanyListReq.numOfEmployeesTo } : null,
                    operatingRevenueFrom: grpcCompanyListReq.operatingRevenueFrom ? { value: grpcCompanyListReq.operatingRevenueFrom } : null,
                    operatingRevenueTo: grpcCompanyListReq.operatingRevenueTo ? { value: grpcCompanyListReq.operatingRevenueTo } : null,
                    marketCapitalizationFrom: grpcCompanyListReq.marketCapitalizationFrom ? { value: grpcCompanyListReq.marketCapitalizationFrom } : null,
                    marketCapitalizationTo: grpcCompanyListReq.marketCapitalizationTo ? { value: grpcCompanyListReq.marketCapitalizationTo } : null,
                    totalAssetsFrom: grpcCompanyListReq.totalAssetsFrom ? { value: grpcCompanyListReq.totalAssetsFrom } : null,
                    totalAssetsTo: grpcCompanyListReq.totalAssetsTo ? { value: grpcCompanyListReq.totalAssetsTo } : null,
                    shareholdersFundsFrom: grpcCompanyListReq.shareholdersFundsFrom ? { value: grpcCompanyListReq.shareholdersFundsFrom } : null,
                    shareholdersFundsTo: grpcCompanyListReq.shareholdersFundsTo ? { value: grpcCompanyListReq.shareholdersFundsTo } : null,
                    pLBeforeTaxFrom: grpcCompanyListReq.pLBeforeTaxFrom ? { value: grpcCompanyListReq.pLBeforeTaxFrom } : null,
                    pLBeforeTaxTo: grpcCompanyListReq.pLBeforeTaxTo ? { value: grpcCompanyListReq.pLBeforeTaxTo } : null,
                    isPrimary: grpcCompanyListReq.isPrimary,
                    aggregation: aggregation,
                    guids: grpcCompanyListReq.guids ? grpcCompanyListReq.guids : [],
                    fields: fields
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.companyList`)
                })
        });
    }


    async companyRetrieve(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.companyRetrieve`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyRetrieve(
                {
                    guid: guid
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.companyRetrieve`)
                })
        });
    }


    async companyListCount(searchStr: string[], isCustomizedPubDate: boolean): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.companyListCount`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyListCount(
                {
                    searchStr: searchStr,
                    isCustomizedPubDate: isCustomizedPubDate
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.companyListCount`)
                })
        });
    }

    async companyRetrieveByIds(guids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.companyRetrieveByIds`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyRetrieveByIds(
                {
                    guids: guids
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.companyRetrieveByIds`)
                })
        });
    }

    async companyPeriodMentions(mentionedCompanies: string[], select: number, start: string, end: string, relatedCount: number, companyCount: number): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.companyPeriodMentions`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyPeriodMentions(
                {
                    mentionedCompanies: mentionedCompanies,
                    select: select,
                    start: start,
                    end: end,
                    relatedCount: relatedCount,
                    companyCount: companyCount
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.companyPeriodMentions`)
                })
        });
    }

    async countryOverview(): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.countryOverview`)
        return new Promise((resolve) => {
            SearchClient.searchServer.countryOverview(
                {
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.countryOverview`)
                })
        });
    }


    async countryDetail(country: string, sector: string): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.countryDetail`)
        return new Promise((resolve) => {
            SearchClient.searchServer.countryDetail(
                {
                    country: country,
                    sector: sector
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.countryDetail`)
                })
        });
    }

    async emisDownload(fileName: string): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.emisDownload`)
        return new Promise((resolve) => {
            SearchClient.searchServer.emisDownload(
                {
                    fileName: fileName
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.emisDownload`)
                })
        });
    }

    async emisList(country: string, sector: string): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.emisList`)
        return new Promise((resolve) => {
            SearchClient.searchServer.emisList(
                {
                    country: country,
                    sector: sector
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.emisList`)
                })
        });
    }

    async trendingTopics(): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.trendingTopics`)
        return new Promise((resolve) => {
            SearchClient.searchServer.trendingTopics(
                {},
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.trendingTopics`)
                })
        });
    }

    async companyFinancial(guid: string, guids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.financial`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyFinancial(
                {
                    guid: guid,
                    guids: guids
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.financial`)
                })
        });
    }

    async companyPeople(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.companyPeople`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyPeople(
                {
                    guid: guid
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.companyPeople`)
                })
        });
    }

    async companyMarket(guid: string, from: string, to: string): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.companyMarket`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyMarket(
                {
                    guid: guid,
                    from: from,
                    to: to
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.companyMarket`)
                })
        });
    }

    async companyPeopleByIds(guids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start SearchClient.companyPeopleByIds`)
        return new Promise((resolve) => {
            SearchClient.searchServer.companyPeopleByIds(
                {
                    guids: guids,
                },
                SearchClient.metaData,
                SearchClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SearchClient.companyPeopleByIds`)
                })
        });
    }
}
