import * as grpc from 'grpc';
import { ClientGRPC } from '../grpc/grpcClient';
import { getKey, logger, UserInfo, generateJWT, commonAppConst } from 'skyeye-svc-common-utils';
import { GrpcClientInstance } from '../interfaces/grpcClientInstance';
import { GrpcServiceResponse } from '../interfaces/grpcServiceResponse';
import { grpcConst } from '../utils/grpcConst';
import { GrpcListRequest } from '../interfaces/grpcListRequest';
const ProtoPath = __dirname + grpcConst.protoPath + grpcConst.proto.subscription;

export class SubscriptionClient {
    private static instance: SubscriptionClient;
    private static subscriptionServer: any;
    private static metaData: grpc.Metadata;
    private static options: grpc.CallOptions;
    private constructor() { }
    public static async getInstance(userInfo: UserInfo): Promise<SubscriptionClient> {
        if (!SubscriptionClient.instance) {
            SubscriptionClient.instance = new SubscriptionClient();
            let clientGrpc: ClientGRPC = new ClientGRPC();
            let clientInit: GrpcClientInstance = await clientGrpc.initClient(ProtoPath);

            let SubscriptionService: any = grpc.loadPackageDefinition(clientInit.loadPackageDefinition);
            SubscriptionService = SubscriptionService.hs[grpcConst.service.subscription].Service;

            let clientConnection = await getKey(grpcConst.hostName.subscription) || grpcConst.defaultGRPCAddress;
            this.subscriptionServer = new SubscriptionService(`${clientConnection}`, grpc.credentials.createInsecure());
            logger.info(`started ${grpcConst.hostName.subscription} connection : ${clientConnection}`);
        }
        SubscriptionClient.metaData = new grpc.Metadata();
        if (userInfo) {
            let token: string = `${commonAppConst.httpHeader.BEARER} ${await generateJWT(userInfo)}`;
            SubscriptionClient.metaData.add(commonAppConst.httpHeader.AUTHORIZATION, token);
        }
        let deadline = Number(await getKey(grpcConst.deadline.short));
        deadline = new Date().setSeconds(new Date().getSeconds() + deadline);
        SubscriptionClient.options = { deadline: deadline };
        return SubscriptionClient.instance;
    }

    async root(): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.root`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.root(
                {},
                SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.root`)
                })
        });
    }

    async createLicense(userGuid: string, subscriptionGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.createLicense`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.createLicense(
                { userGuid: userGuid, subscriptionGuid: subscriptionGuid }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.createLicense`)
                })
        })
    }

    async deleteLicense(licenses: []): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.deleteLicense`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.deleteLicense(
                { licenses: licenses }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.deleteLicense`)
                })
        })
    }

    async deleteLicenseByUserGuid(userGuid: string, organisationGuid: string = null): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.deleteLicenseByUserGuid`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.deleteLicenseByUserGuid(
                { userGuid: userGuid, organisationGuid: organisationGuid }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.deleteLicenseByUserGuid`)
                })
        })
    }

    async createSubscription(subscriptionPlanGuid: string, organisationGuid: string = null, startDate: Date, endDate: Date = null, price: number, subscriptionFeatures: []): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.createSubscription`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.createSubscription(
                { subscriptionPlanGuid: subscriptionPlanGuid, organisationGuid: organisationGuid, startDate: startDate, endDate: endDate, price: price, subscriptionFeatures: subscriptionFeatures }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.createSubscription`)
                })
        })
    }

    async retrieveSubscription(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.retrieveSubscription`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.retrieveSubscription(
                { guid: guid }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.retrieveSubscription`)
                })
        })
    }

    async listSubscription(organisationGuid: string = null, deletedStatus: number = null, listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.listSubscription`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.listSubscription(
                { organisationGuid: organisationGuid, deletedStatus: deletedStatus, ...listProps }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.listSubscription`)
                })
        })
    }

    async deleteSubscription(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.deleteSubscription`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.deleteSubscription(
                { guid: guid }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.deleteSubscription`)
                })
        })
    }

    async retrieveByIdsSubscription(guids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.retrieveByIdsSubscription`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.retrieveByIdsSubscription(
                { guids: guids }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.retrieveByIdsSubscription`)
                })
        })
    }

    async updateSubscription(guid: string, subscriptionPlanGuid: string, organisationGuid: string = null, startDate: Date, endDate: Date = null, price: number, subscriptionFeatures: [], subscriptionName: string = null): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.updateSubscription`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.updateSubscription(
                { guid: guid, subscriptionPlanGuid: subscriptionPlanGuid, organisationGuid: organisationGuid, startDate: startDate, endDate: endDate, price: price, subscriptionFeatures: subscriptionFeatures, subscriptionName: subscriptionName }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.updateSubscription`)
                })
        })
    }

    async getSubscriptionByUserGuid(userGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.getSubscriptionByUserGuid`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.getSubscriptionByUserGuid(
                { userGuid: userGuid }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.getSubscriptionByUserGuid`)
                })
        })
    }

    async getSubscriptionByOrganisationGuid(organisationGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.getSubscriptionByOrganisationGuid`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.getSubscriptionByOrganisationGuid(
                { organisationGuid: organisationGuid }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.getSubscriptionByOrganisationGuid`)
                })
        })
    }

    async createSubscriptionPlan(name: string, subscriptionType: string, description: string, dateTypeNumber: number, dateTypeValue: string, price: number, subscriptionPlanPermissions: string[], subscriptionPlanFeatures: []): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.createSubscriptionPlan`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.createSubscriptionPlan(
                { name: name, subscriptionType: subscriptionType, description: description, dateTypeNumber: dateTypeNumber, dateTypeValue: dateTypeValue, price: price, subscriptionPlanPermissions: subscriptionPlanPermissions, subscriptionPlanFeatures: subscriptionPlanFeatures }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.createSubscriptionPlan`)
                })
        })
    }

    async retrieveSubscriptionPlan(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.retrieveSubscriptionPlan`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.retrieveSubscriptionPlan(
                { guid: guid }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.retrieveSubscriptionPlan`)
                })
        })
    }

    async listSubscriptionPlan(name: string = null, deletedStatus: number = null, listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.listSubscriptionPlan`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.listSubscriptionPlan(
                { name: name, deletedStatus: deletedStatus, ...listProps }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.listSubscriptionPlan`)
                })
        })
    }

    async deleteSubscriptionPlan(guid: string, deletedStatus: number): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.deleteSubscriptionPlan`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.deleteSubscriptionPlan(
                { guid: guid, deletedStatus: deletedStatus }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.deleteSubscriptionPlan`)
                })
        })
    }

    async retrieveByIdsSubscriptionPlan(guids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.retrieveByIdsSubscriptionPlan`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.retrieveByIdsSubscriptionPlan(
                { guids: guids }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.retrieveByIdsSubscriptionPlan`)
                })
        })
    }

    async updateSubscriptionPlan(guid: string, name: string, subscriptionType: string, description: string, dateTypeNumber: number, dateTypeValue: string, price: number, subscriptionPlanPermissions: string[], subscriptionPlanFeatures: []): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.updateSubscriptionPlan`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.updateSubscriptionPlan(
                { guid: guid, name: name, subscriptionType: subscriptionType, description: description, dateTypeNumber: dateTypeNumber, dateTypeValue: dateTypeValue, price: price, subscriptionPlanPermissions: subscriptionPlanPermissions, subscriptionPlanFeatures: subscriptionPlanFeatures }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.updateSubscriptionPlan`)
                })
        })
    }

    async retrieveSubscriptionPlanFeature(): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.retrieveSubscriptionPlanFeature`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.retrieveSubscriptionPlanFeature(
                {}, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.retrieveSubscriptionPlanFeature`)
                })
        })
    }

    async retrieveSubscriptionPlanPermission(): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.retrieveSubscriptionPlanPermission`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.retrieveSubscriptionPlanPermission(
                {}, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.retrieveSubscriptionPlanPermission`)
                })
        })
    }

    async createPersonalSubscription(subscriptionPlanGuid: string, userGuid: string, startDate: string, endDate: string = null, price: number, subscriptionFeatures: []): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.createPersonalSubscription`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.createPersonalSubscription(
                { subscriptionPlanGuid: subscriptionPlanGuid, userGuid: userGuid, startDate: startDate, endDate: endDate = null, price: price, subscriptionFeatures: subscriptionFeatures }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.createPersonalSubscription`)
                })
        })
    }

    async retrieveLicenseByUserGuids(userGuids: string[], deletedStatus: number[], listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.retrieveLicenseByUserGuids`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.retrieveLicenseByUserGuids(
                { userGuids: userGuids, deletedStatus: deletedStatus, ...listProps }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.retrieveLicenseByUserGuids`)
                })
        })
    }

    async changeOrganisationLicense(userGuid: string, fromOrganisationGuid: string, toOrganisationGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start SubscriptionClient.changeOrganisationLicense`)
        return new Promise((resolve) => {
            SubscriptionClient.subscriptionServer.changeOrganisationLicense(
                { userGuid: userGuid, fromOrganisationGuid: fromOrganisationGuid, toOrganisationGuid: toOrganisationGuid }, SubscriptionClient.metaData,
                SubscriptionClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end SubscriptionClient.changeOrganisationLicense`)
                })
        })
    }

}
