import * as grpc from 'grpc';
import { ClientGRPC } from '../grpc/grpcClient';
import { getKey, logger, UserInfo, commonAppConst, generateJWT } from 'skyeye-svc-common-utils';
import { GrpcClientInstance } from '../interfaces/grpcClientInstance';
import { GrpcServiceResponse } from '../interfaces/grpcServiceResponse';
import { grpcConst } from '../utils/grpcConst';
import { GrpcListRequest } from '../interfaces/grpcListRequest';
import { GrpcTargetListEntityModel } from '../models'
const ProtoPath = __dirname + grpcConst.protoPath + grpcConst.proto.user;

export class UserClient {
    private static instance: UserClient;
    private static userServer: any;
    private static metaData: grpc.Metadata;
    private static options: grpc.CallOptions;
    private constructor() { }
    static async getInstance(userInfo: UserInfo): Promise<UserClient> {
        if (!UserClient.instance) {
            UserClient.instance = new UserClient();
            let clientGrpc: ClientGRPC = new ClientGRPC();
            let clientInit: GrpcClientInstance = await clientGrpc.initClient(ProtoPath);

            let UserService: any = grpc.loadPackageDefinition(clientInit.loadPackageDefinition);
            UserService = UserService.hs[grpcConst.service.user].Service;

            let clientConnection = await getKey(grpcConst.hostName.user) || grpcConst.defaultGRPCAddress;
            this.userServer = new UserService(`${clientConnection}`, grpc.credentials.createInsecure());
            logger.info(`started ${grpcConst.hostName.user} connection : ${clientConnection}`);
        }
        UserClient.metaData = new grpc.Metadata();
        if (userInfo) {
            let token: string = `${commonAppConst.httpHeader.BEARER} ${await generateJWT(userInfo)}`;
            UserClient.metaData.add(commonAppConst.httpHeader.AUTHORIZATION, token);
        }
        let deadline = Number(await getKey(grpcConst.deadline.short));
        deadline = new Date().setSeconds(new Date().getSeconds() + deadline);
        UserClient.options = { deadline: deadline };
        return UserClient.instance;
    }

    async root(): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.root`)
        return new Promise((resolve) => {
            UserClient.userServer.root(
                {},
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.root`)
                })
        });
    }

    async accountChangePassword(existPassword: string, newPassword: string, confirmPassword: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.accountChangePassword`)
        return new Promise((resolve) => {
            UserClient.userServer.accountChangePassword(
                {
                    existPassword: existPassword,
                    newPassword: newPassword,
                    confirmPassword: confirmPassword
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.accountChangePassword`)
                })
        });
    }

    async accountForgetPassword(email: string, isPublicUser: boolean): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.accountForgetPassword`)
        return new Promise((resolve) => {
            UserClient.userServer.accountForgetPassword(
                {
                    email: email,
                    isPublicUser: isPublicUser
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.accountForgetPassword`)
                })
        });
    }

    async accountLogin(userName: string, password: string, isPublicUser: boolean): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.accountLogin`)
        return new Promise((resolve) => {
            UserClient.userServer.accountLogin(
                {
                    userName: userName,
                    password: password,
                    isPublicUser: isPublicUser
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.accountLogin`)
                })
        });
    }

    async renewToken(token: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.renewToken`)
        return new Promise((resolve) => {
            UserClient.userServer.renewToken(
                {
                    token: token
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.renewToken`)
                })
        });
    }

    async accountResetPassword(resetKey: string, resetType: string, newPassword: string, confirmPassword: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.accountResetPassword`)
        return new Promise((resolve) => {
            UserClient.userServer.accountResetPassword(
                {
                    resetKey: resetKey,
                    resetType: resetType,
                    newPassword: newPassword,
                    confirmPassword: confirmPassword
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.accountResetPassword`)
                })
        });
    }

    async accountUpdateProfile(firstName: string, lastName: string, email: string, contactNumber: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.accountUpdateProfile`)
        return new Promise((resolve) => {
            UserClient.userServer.accountUpdateProfile(
                {
                    firstName: firstName,
                    lastName: lastName,
                    email: email,
                    contactNumber: contactNumber
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.accountUpdateProfile`)
                })
        });
    }
    async accountRetrieve(): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.accountRetrieve`)
        return new Promise((resolve) => {
            UserClient.userServer.accountRetrieve(
                {},
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.accountRetrieve`)
                })
        });
    }
    async accountSignUp(firstName: string, lastName: string, email: string, contactNumber: string
        , userName: string, isPublicUser: boolean): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.accountSignUp`)
        return new Promise((resolve) => {
            UserClient.userServer.accountSignUp(
                {
                    userName: userName,
                    firstName: firstName,
                    lastName: lastName,
                    email: email,
                    contactNumber: contactNumber,
                    isPublicUser: isPublicUser
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.accountSignUp`)
                })
        });
    }

    async userCreate(firstName: string, lastName: string, email: string, contactNumber: string
        , userName: string, isPublicUser: boolean, organisationGuid?: string, roleList?: string[]
    ): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.userCreate`)
        return new Promise((resolve) => {
            UserClient.userServer.userCreate(
                {
                    userName: userName,
                    firstName: firstName,
                    lastName: lastName,
                    email: email,
                    contactNumber: contactNumber,
                    isPublicUser: isPublicUser,
                    organisationGuid: organisationGuid,
                    roleList: roleList
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.userCreate`)
                })
        });
    }

    async userDelete(guid: string, isPublicUser: boolean, deletedStatus?: number): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.userDelete`)
        return new Promise((resolve) => {
            UserClient.userServer.userDelete(
                {
                    guid: guid,
                    isPublicUser: isPublicUser,
                    deletedStatus: deletedStatus
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.userDelete`)
                })
        });
    }

    async userList(searchStr: string, searchType: string,
        isPublicUser: boolean, listProps: GrpcListRequest, status?: number[]): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.userList`)
        return new Promise((resolve) => {
            UserClient.userServer.userList(
                {
                    searchStr: searchStr,
                    searchType: searchType,
                    status: status,
                    isPublicUser: isPublicUser,
                    ...listProps
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.userList`)
                })
        });
    }

    async userRetrieve(guid: string, isPublicUser: boolean): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.userRetrieve`)
        return new Promise((resolve) => {
            UserClient.userServer.userRetrieve(
                {
                    guid: guid,
                    isPublicUser: isPublicUser
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.userRetrieve`)
                })
        });
    }

    async userRetrieveByIds(guids: string[], deletedStatus: number[], isActive: boolean, isPagination: boolean, grpcListRequest: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.userRetrieveByIds`)
        return new Promise((resolve) => {
            UserClient.userServer.userRetrieveByIds(
                {
                    guids: guids,
                    deletedStatus: deletedStatus,
                    isActive: isActive,
                    isPagination: isPagination,
                    ...grpcListRequest
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.userRetrieveByIds`)
                })
        });
    }

    async licenseUserByIds(guids: string[], deletedStatus: number[], isActive: boolean, grpcListRequest: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.licenseUserByIds`)
        return new Promise((resolve) => {
            UserClient.userServer.licenseUserByIds(
                {
                    guids: guids,
                    deletedStatus: deletedStatus,
                    isActive: isActive,
                    ...grpcListRequest
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.licenseUserByIds`)
                })
        });
    }


    async userUpdate(guid: string, firstName: string, lastName: string, email: string, contactNumber: string
        , userName: string, isPublicUser: boolean, organisationGuid?: string, roleList?: string[]
    ): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.userUpdate`)
        return new Promise((resolve) => {
            UserClient.userServer.userUpdate(
                {
                    guid: guid,
                    userName: userName,
                    firstName: firstName,
                    lastName: lastName,
                    email: email,
                    contactNumber: contactNumber,
                    isPublicUser: isPublicUser,
                    organisationGuid: organisationGuid,
                    roleList: roleList
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.userUpdate`)
                })
        });
    }

    async orgDelinkUser(email: string, isPublicUser: boolean, organisationGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.orgDelinkUser`)
        return new Promise((resolve) => {
            UserClient.userServer.orgDelinkUser(
                {
                    email: email,
                    isPublicUser: isPublicUser,
                    organisationGuid: organisationGuid
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.orgDelinkUser`)
                })
        });
    }

    async orgLinkUser(email: string, isPublicUser: boolean, organisationName: string, organisationGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.orgLinkUser`)
        return new Promise((resolve) => {
            UserClient.userServer.orgLinkUser(
                {
                    email: email,
                    isPublicUser: isPublicUser,
                    organisationGuid: organisationGuid,
                    organisationName: organisationName
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.orgLinkUser`)
                })
        });
    }

    async acceptLinkUser(resetType: string, resetKey: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.acceptLinkUser`)
        return new Promise((resolve) => {
            UserClient.userServer.acceptLinkUser(
                {
                    resetType: resetType,
                    resetKey: resetKey
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.acceptLinkUser`)
                })
        });
    }

    async browseHistoryCreate(entityGuid: string, entityType: string, entityName: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.browseHistoryCreate`)
        return new Promise((resolve) => {
            UserClient.userServer.browseHistoryCreate(
                {
                    entityGuid: entityGuid,
                    entityType: entityType,
                    entityName: entityName
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.browseHistoryCreate`)
                })
        });
    }

    async browseHistoryDelete(guids: string[], userGuid?: string, deletedStatus?: number): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.browseHistoryDelete`)
        return new Promise((resolve) => {
            UserClient.userServer.browseHistoryDelete(
                {
                    guids: guids,
                    userGuid: userGuid,
                    deletedStatus: deletedStatus
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.browseHistoryDelete`)
                })
        });
    }

    async browseHistoryList(entityType: string, entityName: string, listProps: GrpcListRequest,
        userGuid?: string, status?: number[]): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.browseHistoryList`)
        return new Promise((resolve) => {
            UserClient.userServer.browseHistoryList(
                {
                    userGuid: userGuid,
                    status: status,
                    entityType: entityType,
                    entityName: entityName,
                    ...listProps
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.browseHistoryList`)
                })
        });
    }

    async savedSearchCreate(name: string, searchType: string, searchTerm: string, userGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.savedSearchCreate`)
        return new Promise((resolve) => {
            UserClient.userServer.savedSearchCreate(
                {
                    name: name,
                    searchType: searchType,
                    searchTerm: searchTerm,
                    userGuid: userGuid
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.savedSearchCreate`)
                })
        });
    }

    async savedSearchDelete(guid: string, guids: string[], deletedStatus?: number): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.savedSearchDelete`)
        return new Promise((resolve) => {
            UserClient.userServer.savedSearchDelete(
                {
                    guid: guid,
                    guids: guids,
                    deletedStatus: deletedStatus
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.savedSearchDelete`)
                })
        });
    }

    async savedSearchList(searchStr: string, listProps: GrpcListRequest, userGuid?: string, status?: number[]): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.savedSearchList`)
        return new Promise((resolve) => {
            UserClient.userServer.savedSearchList(
                {
                    searchStr: searchStr,
                    userGuid: userGuid,
                    status: status,
                    ...listProps
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.savedSearchList`)
                })
        });
    }

    async savedSearchUpdate(guid: string, name: string, searchType: string, searchTerm: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.savedSearchUpdate`)
        return new Promise((resolve) => {
            UserClient.userServer.savedSearchUpdate(
                {
                    guid: guid,
                    name: name,
                    searchType: searchType,
                    searchTerm: searchTerm
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.savedSearchUpdate`)
                })
        });
    }

    async savedSearchRetrieveByIds(guids: string[], deletedStatus: number[], isPagination: boolean, grpcListRequest: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.savedSearchRetrieveByIds`)
        return new Promise((resolve) => {
            UserClient.userServer.savedSearchRetrieveByIds(
                {
                    guids: guids,
                    deletedStatus: deletedStatus,
                    isPagination: isPagination,
                    ...grpcListRequest
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.savedSearchRetrieveByIds`)
                })
        });
    }

    async searchHistoryCreate(query: string, searchType: string, userGuid: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.searchHistoryCreate`)
        return new Promise((resolve) => {
            UserClient.userServer.searchHistoryCreate(
                {
                    query: query,
                    searchType: searchType,
                    userGuid: userGuid
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.searchHistoryCreate`)
                })
        });
    }

    async searchHistoryDelete(guids: string[], userGuid?: string, deletedStatus?: number): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.searchHistoryDelete`)
        return new Promise((resolve) => {
            UserClient.userServer.searchHistoryDelete(
                {
                    guids: guids,
                    userGuid: userGuid,
                    deletedStatus: deletedStatus
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.searchHistoryDelete`)
                })
        });
    }

    async searchHistoryList(query: string, searchType: string, listProps: GrpcListRequest,
        userGuid?: string, status?: number[]): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.searchHistoryList`)
        return new Promise((resolve) => {
            UserClient.userServer.searchHistoryList(
                {
                    userGuid: userGuid,
                    status: status,
                    query: query,
                    searchType: searchType,
                    ...listProps
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.searchHistoryList`)
                })
        });
    }

    async targetListCreate(name: string, description: string, userGuid: string, entities: GrpcTargetListEntityModel[]): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListCreate`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListCreate(
                {
                    name: name,
                    description: description,
                    entities: entities,
                    userGuid: userGuid
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListCreate`)
                })
        });
    }

    async targetListDelete(guid: string, guids: string[], deletedStatus?: number): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListDelete`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListDelete(
                {
                    guid: guid,
                    guids: guids,
                    deletedStatus: deletedStatus
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListDelete`)
                })
        });
    }

    async targetListList(searchStr: string, listProps: GrpcListRequest, userGuid?: string, status?: number[]): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListList`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListList(
                {
                    searchStr: searchStr,
                    userGuid: userGuid,
                    status: status,
                    ...listProps
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListList`)
                })
        });
    }

    async targetListRetrieve(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListRetrieve`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListRetrieve(
                {
                    guid: guid
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListRetrieve`)
                })
        });
    }

    async targetListRetrieveByIds(guids: string[], deletedStatus: number[], isPagination: boolean, grpcListRequest: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListRetrieveByIds`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListRetrieveByIds(
                {
                    guids: guids,
                    deletedStatus: deletedStatus,
                    isPagination: isPagination,
                    ...grpcListRequest
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListRetrieveByIds`)
                })
        });
    }

    async targetListUpdate(guid: string, name: string, description: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListUpdate`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListUpdate(
                {
                    guid: guid,
                    name: name,
                    description: description
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListUpdate`)
                })
        });
    }

    async targetListItemAdd(targetListGuid: string, targetListGuids: string[]
        , entities: GrpcTargetListEntityModel[], userGuid?: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListItemAdd`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListItemAdd(
                {
                    targetListGuid: targetListGuid,
                    targetListGuids: targetListGuids,
                    entities: entities,
                    userGuid: userGuid
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListItemAdd`)
                })
        });
    }

    async targetListItemDelete(guids: string[], targetListGuid: string, deletedStatus?: number, userGuid?: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListItemDelete`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListItemDelete(
                {
                    guids: guids,
                    targetListGuid: targetListGuid,
                    deletedStatus: deletedStatus,
                    userGuid: userGuid
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListItemDelete`)
                })
        });
    }

    async targetListExport(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListExport`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListExport(
                {
                    guid: guid
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListExport`)
                })
        });
    }

    async targetListMentions(userGuid: string, relatedCount: number, myTargetCount: number, select: number): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.targetListMentions`)
        return new Promise((resolve) => {
            UserClient.userServer.targetListMentions(
                {
                    userGuid: userGuid,
                    relatedCount: relatedCount,
                    myTargetCount: myTargetCount,
                    select: select
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.targetListMentions`)
                })
        });
    }

    async appUserOrganisationStatistic(organisationGuids: string[]): Promise<GrpcServiceResponse> {
        logger.info(`start UserClient.appUserOrganisationStatistic`)
        return new Promise((resolve) => {
            UserClient.userServer.appUserOrganisationStatistic(
                {
                    organisationGuids: organisationGuids,
                },
                UserClient.metaData,
                UserClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end UserClient.appUserOrganisationStatistic`)
                })
        });
    }


}


