import { httpRequest } from 'skyeye-svc-common-utils';
import { ApiTestConfiguration } from './config';
import { apiTestConfigurations } from '../test/config';


/*
    desc:
        apiTestMiddleware is serves as a middleware for api testing,
        which will reroute the gRPC request into api request to the endpoint
    input:
        configuration: configuration based on service aspect
        apiConfiguration: {
            funcName: the name of func, not used in the func
            endpoint: the url of http request
            method: POST method constantly
            headers: request header
        },
        payload: data schema which follows the json format when using http request
    output:
        unresolved promise<any>: a response for http request, which should same as the operation with Grpc    
*/
export function apiTestMiddleware(
    configuration: ApiTestConfiguration,
    apiConfiguration: {
        funcName: string,
        endpoint: string,
        method: string,
        headers: Record<string, string>
    },
    payload: any
) {
    return httpRequest(
        apiConfiguration.method,
        `${configuration.domain}/${configuration.serviceKeyword}/${apiConfiguration.endpoint}`,
        payload,
        apiConfiguration.headers
    );                  
};

/*
    desc:
        constructClientHandler used for constructing a api test handler for gRPC service client,
        that is, cooperating with Proxy, handler will rewrite the prototype of function,
        which reroute to function apiTestMiddleware
    input:
        clientName: the name of the client service name for mapping the configuration
    output:
        function: return a modified function
*/
export function constructClientHandler(clientName: string) {
    return {
        get: function (cls: any, prop: any) {
            return typeof cls[prop] !== "function"
                ? cls[prop]
                : async function (...args: any) {
                    const configuration = apiTestConfigurations.find((v: any) => v.clientName === clientName);
                    const apiConfiguration = configuration.apiConfigurations.find(v => {return v.funcName === prop});
                    const funcArgs = retrieveFuncArgs(cls[prop]);
                    let payload: Record<string, any> = {};
                    for (let i = 0; i < funcArgs.length; ++i) {
                        payload[funcArgs[i]] = args[i];
                    }
                    return await apiTestMiddleware(configuration, apiConfiguration, payload).then(
                        (resp: { json: () => any; }) => resp.json()
                    ).then((data: any) => {
                        return data;
                    });
                };
        },
    };
}

/*
    desc: 
        retrieveFuncArgs as a helper function to retrieve the arguments of a function, that is,
        given a function test(a: string, b: string), outputs will be: ['a', 'b']
    input:
        func: given func to retrieve the argument names.
    output:
        Array<string>: the array of the arguments
*/
export function retrieveFuncArgs(func: Function): Array<string> {
    const STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
    const ARGUMENT_NAMES = /([^\s,]+)/g;
    const fnStr = func.toString().replace(STRIP_COMMENTS, '');
    const result = fnStr.slice(fnStr.indexOf('(')+1, fnStr.indexOf(')')).match(ARGUMENT_NAMES);
    return result ? result : [];
}